@@ -211,20 +211,19 @@ gdbm_avail_traverse (GDBM_FILE dbf,
   size_t size;
   off_t n;
   struct off_map map = OFF_MAP_INITIALIZER;
-  int rc;
+  int rc = 0;
   
   GDBM_ASSERT_CONSISTENCY (dbf, -1);
-  if (gdbm_avail_block_validate (dbf, &dbf->header->avail,
-				 GDBM_HEADER_AVAIL_SIZE (dbf)))
+  if (gdbm_avail_block_validate (dbf, dbf->avail, dbf->avail_size))
     return -1;
 
-  if (off_map_lookup (&map, offsetof (gdbm_file_header, avail)))
+  if (off_map_lookup (&map, GDBM_HEADER_AVAIL_OFFSET (dbf)))
     {
       GDBM_SET_ERRNO (dbf, GDBM_MALLOC_ERROR, FALSE);
       return -1;
     }
       
-  size = ((((size_t)dbf->header->avail.size * sizeof (avail_elem)) >> 1)
+  size = ((((size_t)dbf->avail->size * sizeof (avail_elem)) >> 1)
 	  + sizeof (avail_block));
   blk = malloc (size);
   if (!blk)
@@ -234,40 +233,42 @@ gdbm_avail_traverse (GDBM_FILE dbf,
       return -1;
     }
 
-  rc = 0;
-  n = dbf->header->avail.next_block;
-  while (n)
-    {
-      rc = off_map_lookup (&map, n);
-      if (rc != GDBM_NO_ERROR)
+  if (!(cb && cb (dbf->avail, 0, data)))
+    {  
+      n = dbf->avail->next_block;
+      while (n)
 	{
-	  if (rc == GDBM_CANNOT_REPLACE)
-	    GDBM_SET_ERRNO (dbf, GDBM_BAD_AVAIL, TRUE);
-	  else
-	    GDBM_SET_ERRNO (dbf, rc, FALSE);
-	  rc = -1;
-	  break;
+	  rc = off_map_lookup (&map, n);
+	  if (rc != GDBM_NO_ERROR)
+	    {
+	      if (rc == GDBM_CANNOT_REPLACE)
+		GDBM_SET_ERRNO (dbf, GDBM_BAD_AVAIL, TRUE);
+	      else
+		GDBM_SET_ERRNO (dbf, rc, FALSE);
+	      rc = -1;
+	      break;
+	    }
+	  
+	  if (gdbm_file_seek (dbf, n, SEEK_SET) != n)
+	    {
+	      GDBM_SET_ERRNO (dbf, GDBM_FILE_SEEK_ERROR, FALSE);
+	      rc = -1;
+	      break;
+	    }
+
+	  if (_gdbm_avail_block_read (dbf, blk, size))
+	    {
+	      rc = -1;
+	      break;
+	    }
+	  
+	  if (cb && cb (blk, n, data))
+	    break;
+	  
+	  n = blk->next_block;
 	}
-
-      if (gdbm_file_seek (dbf, n, SEEK_SET) != n)
-	{
-	  GDBM_SET_ERRNO (dbf, GDBM_FILE_SEEK_ERROR, FALSE);
-	  rc = -1;
-	  break;
-	}
-
-      if (_gdbm_avail_block_read (dbf, blk, size))
-	{
-	  rc = -1;
-	  break;
-	}
-
-      if (cb && cb (blk, n, data))
-	break;
-
-      n = blk->next_block;
     }
-
+  
   free (blk);
   off_map_free (&map);
   
