@@ -21,6 +21,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <stdarg.h>
+#include <string.h>
 #include <ctype.h>
 #include <limits.h>
 
@@ -184,6 +185,7 @@ instream_t instream_readline_create (void);
 int interactive (void);
 int input_context_push (instream_t);
 int input_context_pop (void);
+void input_context_drain (void);
 int input_history_size (void);
 const char *input_history_get (int n);
 const char *input_stream_name (void);
@@ -290,7 +292,7 @@ struct command;
 int command_lookup (const char *str, struct locus *loc, struct command **pcmd);
 
 int run_command (struct command *cmd, struct gdbmarglist *arglist);
-void run_last_command (void);
+int run_last_command (void);
 
 struct xdatum;
 void xd_expand (struct xdatum *xd, size_t size);
@@ -344,14 +346,18 @@ extern struct dsegm *dsdef[];
 #define VART_BOOL   1
 #define VART_INT    2
 
-#define VAR_OK           0       /* operation succeeded */
-#define VAR_ERR_NOTSET   1       /* Only for variable_get:
-				    variable is not set */ 
-#define VAR_ERR_NOTDEF   2       /* no such variable */ 
-#define VAR_ERR_BADTYPE  3       /* variable cannot be coerced to the
-				    requested type (software error) */
-#define VAR_ERR_BADVALUE 4       /* Only for variable_set: the value is
-				    not valid for this variable. */
+enum
+  {
+    VAR_OK,           /* operation succeeded */
+    VAR_ERR_NOTSET,   /* Only for variable_get: variable is not set */ 
+    VAR_ERR_NOTDEF,   /* no such variable */ 
+    VAR_ERR_BADTYPE,  /* variable cannot be coerced to the requested type
+		 	 (software error) */
+    VAR_ERR_BADVALUE, /* Only for variable_set: the value is not valid for
+			 this variable. */
+    VAR_ERR_GDBM,     /* GDBM error */
+  };
+
 
 int variable_set (const char *name, int type, void *val);
 int variable_get (const char *name, int type, void **val);
@@ -359,7 +365,16 @@ int variable_unset(const char *name);
 int variable_is_set (const char *name);
 int variable_is_true (const char *name);
 void variable_print_all (FILE *fp);
-
+static inline int
+variable_has_errno (char *varname, int e)
+{
+  return variable_get (varname, VART_INT, (void**)&e) == VAR_OK && e == 1;
+}
+static inline int
+gdbm_error_is_masked (int e)
+{
+  return variable_has_errno ("errormask", e);
+}
 
 int unescape (int c);
 int escape (int c);
@@ -368,7 +383,8 @@ void end_def (void);
 
 int yylex (void);
 int yylex_destroy (void);
-int yyerror (char const *s);
+void yyerror (char const *s);
+#define YYERROR_IS_DECLARED 1
 int yyparse (void);
 
 void lex_trace (int n);
