@@ -34,7 +34,7 @@
 # define YY_YY_GRAM_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
-# define YYDEBUG 1
+# define YYDEBUG 0
 #endif
 #if YYDEBUG
 extern int yydebug;
@@ -59,19 +59,6 @@ extern int yydebug;
     T_WORD = 269
   };
 #endif
-/* Tokens.  */
-#define T_TYPE 258
-#define T_OFF 259
-#define T_PAD 260
-#define T_DEF 261
-#define T_SET 262
-#define T_UNSET 263
-#define T_BOGUS 264
-#define T_CMD 265
-#define T_SHELL 266
-#define T_NUM 267
-#define T_IDENT 268
-#define T_WORD 269
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -92,7 +79,7 @@ union YYSTYPE
   struct { struct dsegm *head, *tail; } dsegmlist;
   struct command *cmd;
 
-#line 96 "gram.h" /* yacc.c:1909  */
+#line 83 "gram.h" /* yacc.c:1909  */
 };
 
 typedef union YYSTYPE YYSTYPE;
