@@ -63,71 +63,15 @@ retrieve_history (char *str)
   return 0;
 }
 
-struct history_param
-{
-  int from;
-  int count;
-};
-  
-int
-input_history_begin (struct handler_param *param, size_t *exp_count)
-{
-  struct history_param *p;
-  int from = 0, count = history_length;
-
-  switch (param->argc)
-    {
-    case 1:
-      if (getnum (&count, param->argv[0]->v.string, NULL))
-	return 1;
-      if (count > history_length)
-	count = history_length;
-      else
-	from = history_length - count;
-      break;
-
-    case 2:
-      if (getnum (&from, param->argv[0]->v.string, NULL))
-	return 1;
-      if (from)
-	--from;
-      if (getnum (&count, param->argv[1]->v.string, NULL))
-	return 1;
-
-      if (count > history_length)
-	count = history_length;
-    }
-  p = emalloc (sizeof *p);
-  p->from = from;
-  p->count = count;
-  param->data = p;
-  if (exp_count)
-    *exp_count = count;
-  return 0;
-}
-
-void
-input_history_handler (struct handler_param *param)
-{
-  struct history_param *p = param->data;
-  int i;
-  HIST_ENTRY **hlist;
-  FILE *fp = param->fp;
-  
-  hlist = history_list ();
-  for (i = 0; i < p->count; i++)
-    fprintf (fp, "%4d) %s\n", p->from + i + 1, hlist[p->from + i]->line);
-}
-
 #define HISTFILE_PREFIX "~/."
 #define HISTFILE_SUFFIX "_history"
 
+static char *history_file_name;
+
 static char *
 get_history_file_name (void)
 {
-  static char *filename = NULL;
-
-  if (!filename)
+  if (!history_file_name)
     {
       char *hname;
 
@@ -136,10 +80,10 @@ get_history_file_name (void)
       strcpy (hname, HISTFILE_PREFIX);
       strcat (hname, rl_readline_name);
       strcat (hname, HISTFILE_SUFFIX);
-      filename = tildexpand (hname);
+      history_file_name = tildexpand (hname);
       free (hname);
     }
-  return filename;
+  return history_file_name;
 }
  
 static char **
@@ -157,26 +101,16 @@ shell_completion (const char *text, int start, int end)
 
   return (matches);
 }
-
-void
-input_init (void)
-{
-  /* Allow conditional parsing of the ~/.inputrc file. */
-  rl_readline_name = (char *) progname;
-  rl_attempted_completion_function = shell_completion;
-  rl_pre_input_hook = pre_input;
-  read_history (get_history_file_name ());
-}
-
-void
-input_done (void)
-{
-  write_history (get_history_file_name ());
-}
 
 static void
-instream_stdin_close (instream_t istr)
+instream_readline_close (instream_t istr)
 {
+  if (history_file_name)
+    {
+      write_history (history_file_name);
+      free (history_file_name);
+      history_file_name = NULL;
+    }
   free (istr);
 }
 
@@ -234,7 +168,7 @@ stdin_read_readline (instream_t istr, char *buf, size_t size)
 } 
 
 static ssize_t
-instream_stdin_read (instream_t istr, char *buf, size_t size)
+instream_readline_read (instream_t istr, char *buf, size_t size)
 {
   if (istr->in_inter)
     return stdin_read_readline (istr, buf, size);
@@ -242,22 +176,48 @@ instream_stdin_read (instream_t istr, char *buf, size_t size)
 }
 
 static int
-instream_stdin_eq (instream_t a, instream_t b)
+instream_readline_eq (instream_t a, instream_t b)
 {
   return 0;
 }
 
-instream_t
-instream_stdin_create (void)
+static int
+instream_readline_history_size (instream_t istr)
 {
+  return history_length;
+}
+
+static const char *
+instream_readline_history_get (instream_t instr, int n)
+{
+  if (n < history_length)
+    return history_list ()[n]->line;
+  return NULL;
+}
+
+instream_t
+instream_readline_create (void)
+{  
   struct instream *istr;
 
-  istr = emalloc (sizeof *istr);
-  istr->in_name = "stdin";
-  istr->in_inter = isatty (fileno (stdin));
-  istr->in_read = instream_stdin_read;
-  istr->in_close = instream_stdin_close;
-  istr->in_eq = instream_stdin_eq;
-
+  if (isatty (fileno (stdin)))
+    {
+      istr = emalloc (sizeof *istr);
+      istr->in_name = "stdin";
+      istr->in_inter = 1;
+      istr->in_read = instream_readline_read;
+      istr->in_close = instream_readline_close;
+      istr->in_eq = instream_readline_eq;
+      istr->in_history_size = instream_readline_history_size;
+      istr->in_history_get = instream_readline_history_get;
+  
+      /* Allow conditional parsing of the ~/.inputrc file. */
+      rl_readline_name = (char *) progname;
+      rl_attempted_completion_function = shell_completion;
+      rl_pre_input_hook = pre_input;
+      read_history (get_history_file_name ());
+    }
+  else
+    istr = instream_stdin_create ();
   return istr;
 }
